<?php declare(strict_types=1);

namespace App\Services\ProjectTranslationServiceTextHash;

use App\Dto\Service\ProjectTranslationServiceTextHash\Codes;
use App\Dto\Service\ProjectTranslationServiceTextHash\TranslateCodes;
use App\Enums\ProjectTranslationServiceHashes\Status;
use App\Models\ProjectTranslationServiceTextHash;
use App\Repositories\ProjectTranslationServiceTextHashRepository;

final readonly class HashTrackerCommand
{
    public function __construct(
        private ProjectTranslationServiceTextHashRepository     $hashRepository,
        private ProjectTranslationServiceTextHashCommandHandler $commandHandler,
    ) { }

    public function execute(array $languages, Codes $codes): TranslateCodes
    {
        $hashes = $this->hashRepository->getHashes($codes->getCodeNames(), $languages)->all();

        $translateCodes = new TranslateCodes();
        foreach ($codes->getCodes() as $code => $value) {
            $textHash = $this->generateHashFromText((string) $value);
            foreach ($languages as $language) {
                $modelHash = $hashes->where('language_id', $language)->firstWhere('code', $code);
                if ($modelHash === null) {
                    $modelHash = $this->commandHandler->handleStore([
                        'language_id' => $language,
                        'code'        => $code,
                        'hash'        => $textHash,
                        'status'      => Status::Waiting,
                    ]);
                    $translateCodes->add($language, $code, $textHash, $modelHash->id);
                    continue;
                }

                if ($modelHash->hash === $textHash && $this->isSuccess($modelHash)) {
                    continue;
                }

                $translateCodes->add($language, $code, $textHash, $modelHash->id);
                $this->commandHandler->handleUpdate($modelHash, [
                    'hash'   => $textHash,
                    'status' => Status::Waiting,
                ]);
            }
        }

        return $translateCodes;
    }

    private function generateHashFromText(string $text): string
    {
        return \hash('sha256', $text);
    }

    private function isSuccess(ProjectTranslationServiceTextHash $modelHash): bool
    {
        if ($modelHash->status === Status::Success) {
            return true;
        }

        return ($modelHash->updated_at >= now()->subMinutes(10));
    }
}
